import os
import threading
import time
import logging
import traceback
from typing import Dict, Any, Callable

import httpx
import redis

from aicp_helper.const import LOG_TAG
from aicp_helper.helper import HelperConfig

PATH = "/kapis/config.kubesphere.io/configs"


class AicpConfig:

    def __init__(self,
                 helper_config: HelperConfig,
                 header: dict[str, str]|None=None,
                 callback: Callable[[dict[str, Any]], None]=None,
                 custom_config: Dict[str, Any]|None=None,
                 config_server=None):
        self.helper_config = helper_config
        self.header = header
        self.custom_config = custom_config
        self.callback = callback
        self.config_server = config_server or "http://config-server-service.aicp-system:8000"

        self.aicp_config = {}
        self.load_or_fresh_aicp_config()

        if not os.getenv("ENV_CONF"):
            threading.Thread(target=self.watch_config).start()


    def load_or_fresh_aicp_config(self):
        self.aicp_config = self.get_aicp_config()
        if self.custom_config:
            self.aicp_config.update(self.custom_config)
        if self.callback:
            self.callback(self.aicp_config)


    def get_aicp_config(self) -> dict[str, Any]|None:
        params = {"svc": [self.helper_config.svc, "common"]}
        retry_times = 10

        while retry_times > 0:
            try:
                ret = httpx.get(self.config_server + PATH, headers=self.header, params=params, timeout=5)
                if ret.status_code == 200:
                    return {item['key']: item['value'] for item in ret.json()["data"]}
                logging.error(f"{LOG_TAG} get config failed [{retry_times}]: [{ret.status_code}][{ret.text}]")
            except Exception as e:
                logging.error(f"{LOG_TAG} get config failed [{retry_times}]: {e}")

            retry_times -= 1
            if retry_times == 0:
                raise Exception("get config failed")
            time.sleep(2)


    def watch_config(self):
        version = 0
        conn = None
        while True:
            try:
                logging.info(f"{LOG_TAG} watch config")
                conn = redis.StrictRedis(host=self.aicp_config['redis_host'], port=self.aicp_config['redis_port'],
                                         password=self.aicp_config['redis_password'], decode_responses=True)
                pubsub = conn.pubsub()
                pubsub.subscribe("notify_config")
                for message in pubsub.listen():
                    if message["type"] == "message":
                        new_version = int(message["data"])
                        if new_version != version:
                            logging.info(f"{LOG_TAG} reload config version [{new_version}]")
                            self.load_or_fresh_aicp_config()
                            version = new_version
            except Exception:  # noqa
                logging.exception(f"{LOG_TAG} watch config failed")
            finally:
                if conn:
                    conn.close()
            time.sleep(10)



from dataclasses import dataclass
from typing import Dict, Any, Callable


@dataclass
class HelperConfig:
    # config
    svc: str = 'common'  # aicp / maas / model / docker / epfs

    # log
    logger = None


class HelperConfigBuilder:

    def __init__(self):
        self.config = {}

    def svc(self, svc: str):
        self.config['svc'] = svc
        return self

    def logger(self, logger):
        self.config['logger'] = logger
        return self

    def build(self):
        return HelperConfig(**self.config)

    def create_helper(self):
        return Helper(self.build())


class Helper:
    def __init__(self, config: HelperConfig):
        self.config = config

    def init_and_watch_config(self,
                              header: dict[str, str],
                              callback: Callable[[dict[str, Any]], None] = None,
                              custom_config: Dict[str, Any]|None=None,
                              config_server: str = None) -> dict[str, Any]:
        from aicp_helper.aicp_config import AicpConfig
        aicp_config = AicpConfig(self.config, header, callback, custom_config, config_server)
        return aicp_config.aicp_config



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
AICP Helper SDK Setup Script
AI Cloud Platform 公共SDK安装配置文件
"""

from setuptools import setup, find_packages
import os


# 读取依赖项
def get_requirements():
    """从 requirements.txt 读取依赖项"""
    requirements_file = "requirements.txt"
    if os.path.exists(requirements_file):
        with open(requirements_file, "r", encoding="utf-8") as f:
            requirements = []
            for line in f:
                line = line.strip()
                if line and not line.startswith("#"):
                    requirements.append(line)
            return requirements
    return []

setup(
    name="aicp-helper",
    version="1.0.3",
    description="AICP Helper SDK - AI Cloud Platform 工具库",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",

    # 作者信息
    author="AICP Team",
    author_email="aicp-team@coreshub.cn",
    maintainer="syw",
    maintainer_email="syw@coreshub.cn",

    # 包配置
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,

    # Python 版本要求
    python_requires=">=3.9",

    # 依赖项
    install_requires=get_requirements(),
)